/*!\file:  Matrix.h
 * \brief wrapper to matrix objects. The goal is to control which API (PETSC,Scalpack, Plapack?) 
 * implements our underlying matrix format.
 */ 

#ifndef _MATRIX_H_
#define _MATRIX_H_

/*Headers:*/
/*{{{1*/
#ifdef HAVE_CONFIG_H
	#include <config.h>
#else
#error "Cannot compile with HAVE_CONFIG_H symbol! run configure first!"
#endif
#include "../Object.h"
#include "../../toolkits/toolkits.h"
#include "../../EnumDefinitions/EnumDefinitions.h"

#ifdef _HAVE_ADOLC_
#include "adolc.h"
#endif

#ifdef _SERIAL_
#include "mex.h"
#endif

/*}}}*/

class Matrix{

	public:
	
		int M,N; 

		#ifdef _HAVE_PETSC_
		Mat matrix; 
		#else
		double* matrix; 
		#endif
		#ifdef _HAVE_ADOLC_
		adouble* amatrix;
		#endif

		/*Matrix constructors, destructors {{{1*/
		Matrix();
		Matrix(int M,int N);
		Matrix(int M,int N,double sparsity);
		Matrix(int M,int N,int connectivity,int numberofdofspernode);
		~Matrix();
		/*}}}*/
		/*Matrix specific routines {{{1*/
		void Echo(void);
		#ifdef _SERIAL_
		mxArray* ToMatlabMatrix(void);
		#endif
		/*}}}*/

};
#endif //#ifndef _MATRIX_H_
